@@ -52,11 +52,9 @@ pub(crate) fn readjust_fn_abi<'tcx>(
             arg.mode = PassMode::Direct(ArgAttributes::new());
         }
 
-        // pass all aggregates directly as values, ptx wants them to be passed all by value, but rustc's
+        // pass all adts directly as values, ptx wants them to be passed all by value, but rustc's
         // ptx-kernel abi seems to be wrong, and it's unstable.
-        if matches!(arg.layout.abi, abi::Abi::Aggregate { .. })
-            && matches!(arg.mode, PassMode::Indirect { .. })
-        {
+        if arg.layout.ty.is_adt() && !matches!(arg.mode, PassMode::Direct { .. }) {
             arg.mode = PassMode::Direct(ArgAttributes::new());
         }
         arg
